/*
 * File:   FileSelector.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */
 
package gui;

import gui.*;
import main.*;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.SpringLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import java.io.*;

/** Class for all types of file selection dialog (open, save-as, new, export)*/
public class FileSelector{

	public static InitConfig config = new InitConfig();
	
	public static File newSimulation(UserInterface ui){
	
		/** Image file filter */
		javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
			public boolean accept(File f) {
				if (f.isDirectory()) return true;
				String name = f.getName().toLowerCase();
				return name.endsWith(".jpeg") || name.endsWith(".jpg")
				|| name.endsWith(".gif") || name.endsWith(".png");
			}

			public String getDescription(){ return "Image (Ariel Photo)"; }
		};

		JFileChooser fc = new JFileChooser(config.getDir());
		AreaChooser ac = new AreaChooser();
		fc.setAccessory(ac);
		fc.addChoosableFileFilter(filter);
		fc.setAcceptAllFileFilterUsed(false);

		int returnVal = fc.showOpenDialog(ui);
		File f = fc.getSelectedFile();
		if(returnVal == JFileChooser.APPROVE_OPTION && f != null) { 
			config.setDir(f.getParent());
			PastureSim.chosenArea = ac.getChosenArea();
			PastureSim.chosenRate = ac.getChosenRate();
			PastureSim.chosenName = ac.getChosenName();
			return f;
		}
		return null;
	}	
	
	
	public static File openSimulation(UserInterface ui){
	
		/** Simulation file filter */
		javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
			public boolean accept(File f) {
				if (f.isDirectory()) return true;
				String name = f.getName().toLowerCase();
				return name.endsWith(".psm");
			}
			public String getDescription(){ return "Simulation"; }
		};				

		JFileChooser fc = new JFileChooser(config.getDir());
		fc.addChoosableFileFilter(filter);
		fc.setAcceptAllFileFilterUsed(false);

		int returnVal = fc.showOpenDialog(ui);
		File f = fc.getSelectedFile();
		if (returnVal == JFileChooser.APPROVE_OPTION && f != null) {
			config.setDir(f.getParent());
			return f;
		}
		return null;
	}

	
	
	public static File saveSimulation(UserInterface ui){
				
		/** Simulation file filter */
		javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
			public boolean accept(File f) {
				if (f.isDirectory()) return true;
				String name = f.getName().toLowerCase();
				return name.endsWith(".psm");
			}
			public String getDescription(){ return "Simulation"; }
		};

		JFileChooser fc = new JFileChooser(config.getDir());
		fc.addChoosableFileFilter(filter);
		fc.setAcceptAllFileFilterUsed(false);

		int returnVal = fc.showSaveDialog(ui);
		File f = fc.getSelectedFile();
		if (returnVal == JFileChooser.APPROVE_OPTION && f != null) {
			config.setDir(f.getParent());
			if(!f.getName().toLowerCase().endsWith(".psm"))
				f = new File(f.getPath() + ".psm");
			return f;
		}
		return null;
	}
	
	
	public static File saveImage(UserInterface ui){
	
		/** Simulation file filter */
		javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter(){
			public boolean accept(File f) {
				if (f.isDirectory()) return true;
				String name = f.getName().toLowerCase();
				return name.endsWith(".png");
			}
			public String getDescription(){ return "PNG Image"; }
		};

		JFileChooser fc = new JFileChooser(config.getDir());
		fc.addChoosableFileFilter(filter);
		fc.setAcceptAllFileFilterUsed(false);

		int returnVal = fc.showSaveDialog(ui);
		File f = fc.getSelectedFile();
		if (returnVal == JFileChooser.APPROVE_OPTION && f != null) {
			config.setDir(f.getParent());
			if(!f.getName().toLowerCase().endsWith(".png"))
				f = new File(f.getPath() + ".png");
			return f;
		}
		return null;
	
	}	

}
