/*
 * File:   MenuBar.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */
 
package gui;

import gui.icon.*;
import main.PastureSim;

import sim.Simulation;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/** The main menu bar */
public class MenuBar extends JMenuBar {

	private Simulation sim; // A reference to the simulation
	private UserInterface ui;

	//The menu items to add action listners to
	private JMenuItem simNew;
	private JMenuItem simOpen;
	private JMenuItem simSave;
	private JMenuItem simSaveAs;
	private JMenuItem simExport;
	private JMenuItem simClose;
	private JMenuItem simExit;
	
	private JRadioButtonMenuItem modeAdjust;
	private JRadioButtonMenuItem modeSimulate; 
	private JRadioButtonMenuItem modePasture;  
	private JRadioButtonMenuItem modeLivestock;
	private JRadioButtonMenuItem modeHistory;
//private JRadioButtonMenuItem modeGraphs;  

	
	private JMenuItem zoomPlus;
	private JMenuItem zoomMinus;
	
	private JRadioButtonMenuItem viewNoAnimals;
	private JRadioButtonMenuItem viewAnimalNormal;
	private JRadioButtonMenuItem viewAnimalHunger;
	private JRadioButtonMenuItem viewAnimalThirst;
	private JCheckBoxMenuItem viewDead;

	private JCheckBoxMenuItem viewAriel;
	private JCheckBoxMenuItem viewField;
	private JCheckBoxMenuItem viewWater;
	private JRadioButtonMenuItem viewNitrogen;
	private JRadioButtonMenuItem viewCapacity;
	private JRadioButtonMenuItem viewMoisture;
	private JRadioButtonMenuItem viewHeight;
	private JRadioButtonMenuItem viewLeaf;
	private JRadioButtonMenuItem viewFrequency;
	private JRadioButtonMenuItem viewWaterProx;
	private JRadioButtonMenuItem viewNone;
	
	private JMenuItem viewAnimalNutrition;
	private JMenuItem viewPastureState;
	private JMenuItem viewAnimalBehaviour;

	
	public MenuBar(Simulation sim, UserInterface ui) {
		
		this.sim = sim;
		this.ui = ui;
		Dimension size;
		if(sim == null) size = new Dimension(UserInterface.INIT_WIDTH, 20);
		else size = new Dimension(sim.getPastureSize().width, 20);
		setPreferredSize(size);
	
		// The Default menu listner
		MenuActionListener menuListener = new MenuActionListener();
      
		// Menu bar
		JMenu simMenu  = new JMenu("File");
		JMenu modeMenu = new JMenu("Mode");
		JMenu viewMenu = new JMenu("View");

		// Simulation menu
		simNew    = new JMenuItem("New...",     'N'); simMenu.add(simNew);    simNew.addActionListener(menuListener);
		simOpen   = new JMenuItem("Open...",    'O'); simMenu.add(simOpen);   simOpen.addActionListener(menuListener);
		simSave   = new JMenuItem("Save",       'S'); simMenu.add(simSave);   simSave.addActionListener(menuListener);
		simSaveAs = new JMenuItem("Save As...", 'A'); simMenu.add(simSaveAs); simSaveAs.addActionListener(menuListener);
		simExport = new JMenuItem("Export...",  'E'); simMenu.add(simExport); simExport.addActionListener(menuListener);
		simClose  = new JMenuItem("Close",      'C'); simMenu.add(simClose);  simClose.addActionListener(menuListener);
		simExit   = new JMenuItem("Exit",       'X'); simMenu.add(simExit);   simExit.addActionListener(menuListener);
		simMenu.insertSeparator(1);
		simMenu.insertSeparator(6);
		
				
		// Mode menu
		ButtonGroup modeButtons = new ButtonGroup();
		modeAdjust    = new JRadioButtonMenuItem("Adjust Parameters"); modeMenu.add(modeAdjust);    modeButtons.add(modeAdjust);    modeAdjust.addActionListener(menuListener);
		modeSimulate  = new JRadioButtonMenuItem("Simulate");          modeMenu.add(modeSimulate);  modeButtons.add(modeSimulate);  modeSimulate.addActionListener(menuListener);
		modePasture   = new JRadioButtonMenuItem("Pasture Edit");      modeMenu.add(modePasture);   modeButtons.add(modePasture);   modePasture.addActionListener(menuListener);
		modeLivestock = new JRadioButtonMenuItem("Livestock Edit");    modeMenu.add(modeLivestock); modeButtons.add(modeLivestock); modeLivestock.addActionListener(menuListener);
		modeHistory   = new JRadioButtonMenuItem("Histroy");           modeMenu.add(modeHistory);   modeButtons.add(modeHistory);   modeHistory.addActionListener(menuListener);
//modeGraphs    = new JRadioButtonMenuItem("Graphs");            modeMenu.add(modeGraphs);    modeButtons.add(modeGraphs);    modeGraphs.addActionListener(menuListener);
		modeMenu.insertSeparator(2);
		modeMenu.insertSeparator(5);

			
		// View menu
		JMenu viewLivestockMenu = new JMenu("Livestock Views");
		JMenu viewPastureMenu   = new JMenu("Pasture Views");
		JMenu viewGraphMenu     = new JMenu("Graphs");
		
		zoomPlus  = new JMenuItem("Zoom In", new PlusIcon());         viewMenu.add(zoomPlus);  zoomPlus.addActionListener(menuListener);
		zoomMinus = new JMenuItem("Zoom Out", new MinusIcon());       viewMenu.add(zoomMinus); zoomMinus.addActionListener(menuListener);
		
		viewMenu.add(viewLivestockMenu);
		viewMenu.add(viewPastureMenu);
		viewMenu.add(viewGraphMenu);
		viewMenu.insertSeparator(2);
		
		// Livestock view menu
		ButtonGroup viewAnimalButtons = new ButtonGroup();
		viewNoAnimals    = new JRadioButtonMenuItem("Hide Animals");       viewLivestockMenu.add(viewNoAnimals);    viewAnimalButtons.add(viewNoAnimals);    viewNoAnimals.addActionListener(menuListener);
		viewAnimalNormal = new JRadioButtonMenuItem("Show Animals");       viewLivestockMenu.add(viewAnimalNormal); viewAnimalButtons.add(viewAnimalNormal); viewAnimalNormal.addActionListener(menuListener);
		viewAnimalHunger = new JRadioButtonMenuItem("Show Animal Hunger"); viewLivestockMenu.add(viewAnimalHunger); viewAnimalButtons.add(viewAnimalHunger); viewAnimalHunger.addActionListener(menuListener);
		viewAnimalThirst = new JRadioButtonMenuItem("Show Animal Thirst"); viewLivestockMenu.add(viewAnimalThirst); viewAnimalButtons.add(viewAnimalThirst); viewAnimalThirst.addActionListener(menuListener);
		viewDead  = new JCheckBoxMenuItem("Show Dead Livestock", false); viewLivestockMenu.add(viewDead);  viewDead.addActionListener(menuListener);
		viewLivestockMenu.insertSeparator(4);
		
		// Pasture view menu
		viewAriel = new JCheckBoxMenuItem("Show Aerial Photo", false); viewPastureMenu.add(viewAriel); viewAriel.addActionListener(menuListener);
		viewField = new JCheckBoxMenuItem("Show Field Bounds", false); viewPastureMenu.add(viewField); viewField.addActionListener(menuListener);
		viewWater = new JCheckBoxMenuItem("Show Water Paths",  false); viewPastureMenu.add(viewWater); viewWater.addActionListener(menuListener);
		ButtonGroup viewButtons = new ButtonGroup();
		viewNitrogen    = new JRadioButtonMenuItem("Show Soil Nitrogen");   viewPastureMenu.add(viewNitrogen);  viewButtons.add(viewNitrogen);   viewNitrogen.addActionListener(menuListener);
		viewCapacity    = new JRadioButtonMenuItem("Show Water Capacity");  viewPastureMenu.add(viewCapacity);  viewButtons.add(viewCapacity);   viewCapacity.addActionListener(menuListener);
		viewMoisture    = new JRadioButtonMenuItem("Show Soil Moisture");   viewPastureMenu.add(viewMoisture);  viewButtons.add(viewMoisture);   viewMoisture.addActionListener(menuListener);
		viewHeight      = new JRadioButtonMenuItem("Show Biomass Weight");  viewPastureMenu.add(viewHeight);    viewButtons.add(viewHeight);     viewHeight.addActionListener(menuListener);
		viewLeaf        = new JRadioButtonMenuItem("Show Leaf Area");       viewPastureMenu.add(viewLeaf);      viewButtons.add(viewLeaf);       viewLeaf.addActionListener(menuListener);
		viewFrequency   = new JRadioButtonMenuItem("Show Visit Frequency"); viewPastureMenu.add(viewFrequency); viewButtons.add(viewFrequency);  viewFrequency.addActionListener(menuListener);
		viewWaterProx   = new JRadioButtonMenuItem("Show Water Proximity"); viewPastureMenu.add(viewWaterProx); viewButtons.add(viewWaterProx);  viewWaterProx.addActionListener(menuListener);
		viewNone        = new JRadioButtonMenuItem("Show No Data");         viewPastureMenu.add(viewNone);      viewButtons.add(viewNone);       viewNone.addActionListener(menuListener);
		viewPastureMenu.insertSeparator(3);

		// Graphs view menu
		viewAnimalNutrition = new JMenuItem("Animal Nutrition"); viewGraphMenu.add(viewAnimalNutrition); viewAnimalNutrition.addActionListener(menuListener);
		viewPastureState    = new JMenuItem("Pasture State");    viewGraphMenu.add(viewPastureState);       viewPastureState.addActionListener(menuListener);
		viewAnimalBehaviour = new JMenuItem("Animal Behaviour"); viewGraphMenu.add(viewAnimalBehaviour); viewAnimalBehaviour.addActionListener(menuListener);
		
		// Add all menus to menu bar
		add(simMenu);
		add(modeMenu);
		add(viewMenu);
		
		if(sim == null) disableSimulationItems();
		else enableItems();
	}
	
	public void setSimulation(Simulation sim){
		this.sim = sim;
		setPreferredSize(new Dimension(sim.getPastureSize().width, 20));
		viewAriel.setSelected(sim.viewingAriel());
		viewField.setSelected(sim.viewingFields());
		viewDead.setSelected(sim.viewingDeadstock());
		viewWater.setSelected(sim.viewingWaterPaths());
		enableItems();
	}
		
	public void enableItems(){
	
		simNew.setEnabled(true);
		simOpen.setEnabled(true);
		simSave.setEnabled(true);
		simSaveAs.setEnabled(true);
		simExport.setEnabled(true);
		simClose.setEnabled(true);
		simExit.setEnabled(true);

		modeAdjust.setEnabled(true);    modeAdjust.setSelected(sim.getMode() == Simulation.MODE_ADJUST);
		modeSimulate.setEnabled(true);  modeSimulate.setSelected(sim.getMode() == Simulation.MODE_SIMULATE);
		modePasture.setEnabled(true);   modePasture.setSelected(sim.getMode() >= Simulation.MODE_PASTURE && sim.getMode() <= Simulation.MODE_PASTURE_REMOVE_WATER);
		modeLivestock.setEnabled(true); modeLivestock.setSelected(sim.getMode() >= Simulation.MODE_LIVESTOCK && sim.getMode() <= Simulation.MODE_LIVESTOCK_REMOVE);
		modeHistory.setEnabled(true);   modeHistory.setSelected(sim.getMode() == Simulation.MODE_HISTORY);
//modeGraphs.setEnabled(true);    modeGraphs.setSelected(sim.getMode() == Simulation.MODE_GRAPH);

		zoomPlus.setEnabled(sim.canZoomIn());
		zoomMinus.setEnabled(sim.canZoomOut());
		
		viewNoAnimals.setEnabled(true);   viewNoAnimals.setSelected(sim.showingNoAnimals());
		viewAnimalNormal.setEnabled(true);viewAnimalNormal.setSelected(sim.showingAnimalNormal());
		viewAnimalHunger.setEnabled(true);viewAnimalHunger.setSelected(sim.showingAnimalHunger());
		viewAnimalThirst.setEnabled(true);viewAnimalThirst.setSelected(sim.showingAnimalThirst());
		viewDead.setEnabled(true);  viewDead.setSelected(sim.viewingDeadstock());
		
		viewAriel.setEnabled(true); viewAriel.setSelected(sim.viewingAriel());
		viewField.setEnabled(true); viewField.setSelected(sim.viewingFields());
		viewWater.setEnabled(true); viewWater.setSelected(sim.viewingWaterPaths());
		viewNitrogen.setEnabled(true);  viewNitrogen.setSelected(sim.showingNitrogen());
		viewCapacity.setEnabled(true);  viewCapacity.setSelected(sim.showingCapacity());
		viewMoisture.setEnabled(true);  viewMoisture.setSelected(sim.showingMoisture());
		viewHeight.setEnabled(true);    viewHeight.setSelected(sim.showingHeight());
		viewLeaf.setEnabled(true);      viewLeaf.setSelected(sim.showingLeaf());
		viewFrequency.setEnabled(true); viewFrequency.setSelected(sim.showingFrequency());
		viewWaterProx.setEnabled(true); viewWaterProx.setSelected(sim.showingProximity());
		viewNone.setEnabled(true);      viewNone.setSelected(sim.showingNone());
		
		viewAnimalNutrition.setEnabled( ui.nutritionWindow == null);
		viewPastureState.setEnabled(    ui.pastureWindow   == null);
		viewAnimalBehaviour.setEnabled( ui.behaviourWindow == null);
	}
	
	public void disableSimulationItems(){
		simSave.setEnabled(false);
		simSaveAs.setEnabled(false);
		simExport.setEnabled(false);
		simClose.setEnabled(false);

		modeAdjust.setEnabled(false); 
		modeSimulate.setEnabled(false); 
		modePasture.setEnabled(false);  
		modeLivestock.setEnabled(false);
		modeHistory.setEnabled(false);
//modeGraphs.setEnabled(false);  

		zoomPlus.setEnabled(false);
		zoomMinus.setEnabled(false);
		
		viewNoAnimals.setEnabled(false);
		viewAnimalNormal.setEnabled(false);
		viewAnimalHunger.setEnabled(false);
		viewAnimalThirst.setEnabled(false);
		viewDead.setEnabled(false);  viewDead.setSelected(false);
		
		viewAriel.setEnabled(false); viewAriel.setSelected(false);
		viewField.setEnabled(false); viewField.setSelected(false);
		viewWater.setEnabled(false); viewWater.setSelected(false);
		viewNitrogen.setEnabled(false);
		viewCapacity.setEnabled(false);
		viewMoisture.setEnabled(false);
		viewHeight.setEnabled(false); 
		viewLeaf.setEnabled(false); 
		viewFrequency.setEnabled(false); 
		viewWaterProx.setEnabled(false); 
		viewNone.setEnabled(false);
		
		viewAnimalNutrition.setEnabled(false);
		viewPastureState.setEnabled(false);
		viewAnimalBehaviour.setEnabled(false);
	}

	class MenuActionListener implements ActionListener{
	
		public void actionPerformed(ActionEvent e) {

			     if(e.getSource() == simNew)          PastureSim.newSimulation(ui); 
			else if(e.getSource() == simOpen)         PastureSim.openSimulation(ui);
			else if(e.getSource() == simExit)         PastureSim.exit();
			
			if(sim != null){
		
				     if(e.getSource() == simSave)         sim.save();
				else if(e.getSource() == simSaveAs)       sim.saveAs();
				else if(e.getSource() == simExport)       sim.export();
				else if(e.getSource() == simClose)        sim.close();

				else if(e.getSource() == modeAdjust)      sim.setMode(Simulation.MODE_ADJUST);
				else if(e.getSource() == modeSimulate)    sim.setMode(Simulation.MODE_SIMULATE);
				else if(e.getSource() == modePasture)     sim.setMode(Simulation.MODE_PASTURE);
				else if(e.getSource() == modeLivestock)   sim.setMode(Simulation.MODE_LIVESTOCK);
				else if(e.getSource() == modeHistory)     sim.setMode(Simulation.MODE_HISTORY);
//else if(e.getSource() == modeGraphs)      sim.setMode(Simulation.MODE_GRAPH);

				else if(e.getSource() == zoomPlus)        sim.plusZoom();
				else if(e.getSource() == zoomMinus)       sim.minusZoom();
				
				else if(e.getSource() == viewNoAnimals)    sim.showAnimalData(Simulation.DATA_NONE);
				else if(e.getSource() == viewAnimalNormal) sim.showAnimalData(Simulation.DATA_NORMAL);
				else if(e.getSource() == viewAnimalHunger) sim.showAnimalData(Simulation.DATA_HUNGER);
				else if(e.getSource() == viewAnimalThirst) sim.showAnimalData(Simulation.DATA_THIRST);
				else if(e.getSource() == viewDead)         sim.toggleViewDeadstock();
				
				else if(e.getSource() == viewAriel)       sim.toggleViewAriel();
				else if(e.getSource() == viewField)       sim.toggleViewFields();
				else if(e.getSource() == viewWater)       sim.toggleViewWaterPaths();
				else if(e.getSource() == viewNitrogen)    sim.showData(Simulation.DATA_NITROGEN);
				else if(e.getSource() == viewCapacity)    sim.showData(Simulation.DATA_CAPACITY);
				else if(e.getSource() == viewMoisture)    sim.showData(Simulation.DATA_MOISTURE);
				else if(e.getSource() == viewHeight)      sim.showData(Simulation.DATA_HEIGHT);
				else if(e.getSource() == viewLeaf)        sim.showData(Simulation.DATA_LEAF);
				else if(e.getSource() == viewFrequency)   sim.showData(Simulation.DATA_FREQUENCY);
				else if(e.getSource() == viewWaterProx)   sim.showData(Simulation.DATA_WATER_DIST);
				else if(e.getSource() == viewNone)        sim.showData(Simulation.DATA_NONE);
			
				else if(e.getSource() == viewAnimalNutrition) ui.graphNutrition();
				else if(e.getSource() == viewPastureState)    ui.graphPasture();
				else if(e.getSource() == viewAnimalBehaviour) ui.graphBehaviour();
			}
		}
	}
}
