/*
 * File:   ContextPanel.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */
 
package gui.context;

import sim.Simulation;
import gui.UserInterface;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.awt.Color;
import java.awt.Container;
import javax.swing.SpringLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/** All subclasses of this provide the context panel at the bottom of the screen */
public abstract class ContextPanel extends JPanel {

	protected Simulation sim;
	protected SpringLayout layout;

	public ContextPanel(Simulation sim){
		super();
		this.sim = sim;
		layout = new SpringLayout();
		setLayout(layout);
		Dimension size;
		if(sim == null) size = new Dimension(gui.UserInterface.INIT_WIDTH, UserInterface.CONTEXT_HEIGHT);
		else size = new Dimension(sim.getPastureSize().width, UserInterface.CONTEXT_HEIGHT);
		setPreferredSize(size);
		generate();
		if(sim != null) enableButtons();
	}
	
	public void setSimulation(Simulation sim){
		this.sim = sim;
		setPreferredSize(new Dimension(sim.getPastureSize().width, UserInterface.CONTEXT_HEIGHT));
		enableButtons();
	}

	public abstract void generate();
	
	public abstract void enableButtons();
	
	public abstract void disableButtons();
}
