/*
 * File:   HistoryContextPanel.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */

package gui.context;

import gui.icon.*;
import sim.Simulation;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.awt.Color;
import java.awt.Container;
import javax.swing.SpringLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/** Context panel for history browsing mode */
public class HistoryContextPanel extends ContextPanel{


	JButton historyStart;
	JButton historyJumpBack;
	JButton historyBack;
	JLabel  historyDate;
	JLabel  historyNumber;
	JButton historyForward;
	JButton historyJumpForward;
	JButton historyEnd;

	public HistoryContextPanel(Simulation sim){
		super(sim);
	}
	
	public void generate(){

		// The default context button listner
		ButtonListner buttonListner = new ButtonListner();
		
		historyStart       = new JButton(new StartIcon());                  historyStart.setMargin(new Insets(1,5,1,5));    add(historyStart);       historyStart.addActionListener(buttonListner);    // historyStart.setPreferredSize       (new Dimension(29,19)); 
		historyJumpBack    = new JButton(new JumpLeftIcon());               historyJumpBack.setMargin(new Insets(1,0,1,0)); add(historyJumpBack);    historyJumpBack.addActionListener(buttonListner); // historyJumpBack.setPreferredSize    (new Dimension(29,19)); 
		historyBack        = new JButton(new LeftIcon());                   historyBack.setMargin(new Insets(1,5,1,5));     add(historyBack);        historyBack.addActionListener(buttonListner);     // historyBack.setPreferredSize        (new Dimension(29,19)); 
		historyDate        = new JLabel(sim.state.getReadableTime());                                                       add(historyDate);
		historyNumber      = new JLabel("("+sim.getHistoryNumber()+" of "+sim.getHistoryCount()+")"); add(historyNumber);
		historyForward     = new JButton(new RightIcon());                  historyForward.setMargin(new Insets(1,5,1,5));     add(historyForward);     historyForward.addActionListener(buttonListner);    // historyForward.setPreferredSize     (new Dimension(29,19)); 
		historyJumpForward = new JButton(new JumpRightIcon());              historyJumpForward.setMargin(new Insets(1,0,1,0)); add(historyJumpForward); historyJumpForward.addActionListener(buttonListner);// historyJumpForward.setPreferredSize (new Dimension(29,19)); 
		historyEnd         = new JButton(new EndIcon());                    historyEnd.setMargin(new Insets(1,5,1,5));         add(historyEnd);         historyEnd.addActionListener(buttonListner);        //  historyEnd.setPreferredSize         (new Dimension(29,19)); 

		layout.putConstraint(SpringLayout.NORTH, historyDate,         0, SpringLayout.NORTH, this);
		layout.putConstraint(SpringLayout.NORTH, historyNumber,       0, SpringLayout.NORTH, this);
		
		layout.putConstraint(SpringLayout.NORTH, historyStart,        5, SpringLayout.SOUTH, historyDate);
		layout.putConstraint(SpringLayout.NORTH, historyJumpBack,     5, SpringLayout.SOUTH, historyDate);
		layout.putConstraint(SpringLayout.NORTH, historyBack,         5, SpringLayout.SOUTH, historyDate);
		layout.putConstraint(SpringLayout.NORTH, historyForward,      5, SpringLayout.SOUTH, historyDate);
		layout.putConstraint(SpringLayout.NORTH, historyJumpForward,  5, SpringLayout.SOUTH, historyDate);
		layout.putConstraint(SpringLayout.NORTH, historyEnd,          5, SpringLayout.SOUTH, historyDate);

		layout.putConstraint(SpringLayout.EAST, historyDate,   (int)(getPreferredSize().getWidth()/2), SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.WEST, historyNumber,     15, SpringLayout.EAST, historyDate);
		
		layout.putConstraint(SpringLayout.EAST, historyStart,       1, SpringLayout.WEST, historyJumpBack);
		layout.putConstraint(SpringLayout.EAST, historyJumpBack,    1, SpringLayout.WEST, historyBack);
		layout.putConstraint(SpringLayout.EAST, historyBack,        0, SpringLayout.EAST, historyDate);
		layout.putConstraint(SpringLayout.WEST, historyForward,    -1, SpringLayout.EAST, historyDate);
		layout.putConstraint(SpringLayout.WEST, historyJumpForward,-1, SpringLayout.EAST, historyForward);
		layout.putConstraint(SpringLayout.WEST, historyEnd,        -1, SpringLayout.EAST, historyJumpForward);
		sim.gotoState(Simulation.STATE_CURRENT);
		enableButtons();
	}
	
	public void enableButtons(){
		historyStart.setEnabled(      sim.canViewHistory(Simulation.STATE_START));
		historyJumpBack.setEnabled(   sim.canViewHistory(Simulation.STATE_BACK));
		historyBack.setEnabled(       sim.canViewHistory(Simulation.STATE_BACK));
		historyDate.setText(          sim.state.getReadableTime());
		historyNumber.setText(    "("+sim.getHistoryNumber()+" of "+sim.getHistoryCount()+")");
		historyForward.setEnabled(    sim.canViewHistory(Simulation.STATE_FORWARD));
		historyJumpForward.setEnabled(sim.canViewHistory(Simulation.STATE_FORWARD));
		historyEnd.setEnabled(        sim.canViewHistory(Simulation.STATE_CURRENT));
	}
	
	public void disableButtons(){
		historyStart.setEnabled(false);
		historyJumpBack.setEnabled(false);
		historyBack.setEnabled(false);
		historyForward.setEnabled(false);
		historyJumpForward.setEnabled(false);
		historyEnd.setEnabled(false);
	}
	
	
	private class ButtonListner implements ActionListener{
		public void actionPerformed(ActionEvent e) {
			     if(e.getSource() == historyStart)       sim.gotoState(Simulation.STATE_START);
			else if(e.getSource() == historyJumpBack)    sim.gotoState(Simulation.STATE_JBACK);
			else if(e.getSource() == historyBack)        sim.gotoState(Simulation.STATE_BACK);
			else if(e.getSource() == historyForward)     sim.gotoState(Simulation.STATE_FORWARD);
			else if(e.getSource() == historyJumpForward) sim.gotoState(Simulation.STATE_JFORWARD);
			else if(e.getSource() == historyEnd)         sim.gotoState(Simulation.STATE_CURRENT);
			else return;
			sim.refreshViewPanel();
			sim.refreshContextPanel();
		}
	}
}
