/*
 * File:   LivestockSelectDialog.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */

 
package gui.context;

import sim.*;
import sim.livestock.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/** The dialog for selecting livestock groups */
public class LivestockSelectDialog extends JDialog implements ActionListener{

	public static final int PLACE = 0;
	public static final int REMOVE = 1;
	
	private JButton[] groups;
	private LinkedList animals;
	private JFrame parent;
	
	private LivestockGroup group = null;
	
	public LivestockSelectDialog(JFrame parent){ super(parent, true); this.parent = parent; }
	
	public LivestockGroup show(int action, Simulation sim){	
	
		setTitle("Select Group");
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setResizable(false);
		
		Container pane = getContentPane();
		SpringLayout layout = new SpringLayout();
		pane.setLayout(layout);
		
		animals = sim.getLivestock();
		groups = new JButton[animals.size()];
		for(ListIterator i = animals.listIterator(); i.hasNext();){
			groups[i.nextIndex()] = new JButton(((LivestockGroup)i.next()).getName());
			groups[i.previousIndex()].setPreferredSize(new Dimension(150,20));
			groups[i.previousIndex()].addActionListener(this);
			pane.add(groups[i.previousIndex()]);
		}
		
		layout.putConstraint(SpringLayout.NORTH, groups[0], 10, SpringLayout.NORTH, pane);
		layout.putConstraint(SpringLayout.WEST,  groups[0], 10, SpringLayout.WEST, pane);
		layout.putConstraint(SpringLayout.EAST,  pane, 10, SpringLayout.EAST, groups[0]);
		layout.putConstraint(SpringLayout.SOUTH, pane, 10, SpringLayout.SOUTH, groups[groups.length-1]);

		for(int i = 1; i < groups.length; i++){
			layout.putConstraint(SpringLayout.NORTH, groups[i], 10, SpringLayout.SOUTH, groups[i-1]);
			layout.putConstraint(SpringLayout.WEST, groups[i], 0, SpringLayout.WEST, groups[0]);
			layout.putConstraint(SpringLayout.EAST, groups[i], 0, SpringLayout.EAST, groups[0]);
		}

		setLocationRelativeTo(parent);
		pack();
		show();
		
		return group;
	}
	
	public void actionPerformed(ActionEvent e){
		String groupName = ((JButton)e.getSource()).getText();
		
		for(Iterator i = animals.iterator(); i.hasNext();){
			LivestockGroup g = (LivestockGroup)i.next();
			if(g.getName().equals(groupName)){ group = g; break; }
		}
		dispose();
	}	
}
