/*
 * File:   SimulationContextPanel.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */

 
package gui.context;

import gui.icon.*;
import sim.Simulation;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.awt.Color;
import java.awt.Container;
import javax.swing.SpringLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


/** This is the main context panel which controls
  * the actual simulation as it runs.*/
public class SimulationContextPanel extends ContextPanel{

	private JLabel speed;
	private JButton speedMinus;
	private JTextField speedVal;
	private JButton speedPlus;
	private JButton speedMax;
	
	private JLabel time;

	private JLabel zoom;
	private JButton zoomMinus;
	private JTextField zoomVal;
	private JButton zoomPlus;
	
	public SimulationContextPanel(Simulation sim){ super(sim); }
	
	public void generate(){
	
		// The default context button listner
		buttonListener buttonListener = new buttonListener();
		
		speed        = new JLabel("Speed");             add(speed);
		speedMinus   = new JButton(new MinusIcon()); speedMinus.setMargin(new Insets(1,0,1,0)); add(speedMinus); speedMinus.addActionListener(buttonListener); // speedMinus.setPreferredSize(new Dimension(19,19));
		speedVal     = new JTextField(""+sim.getSpeed(), 3); add(speedVal); speedVal.setEditable(false);
		speedPlus    = new JButton(new PlusIcon());  speedPlus.setMargin(new Insets(1,0,1,0)); add(speedPlus); speedPlus.addActionListener(buttonListener); // speedPlus.setPreferredSize(new Dimension(19,19)); 
		speedMax     = new JButton("max"); speedMax.setMargin(new Insets(-1,0,-1,0)); add(speedMax); speedMax.addActionListener(buttonListener);
		
		JLabel timeLabel = new JLabel("Time"); add(timeLabel);
		time        = new JLabel(sim.state.getReadableTime()); time.setFont(new Font(null, Font.PLAIN, 12)); add(time);

		zoom        = new JLabel("Zoom");             add(zoom);
		zoomMinus  = new JButton(new MinusIcon()); zoomMinus.setMargin(new Insets(1,0,1,0)); add(zoomMinus); zoomMinus.addActionListener(buttonListener); //zoomMinus.setPreferredSize(new Dimension(19,19)); 
		zoomVal = new JTextField(""+sim.getZoom(), 3); add(zoomVal); zoomVal.setEditable(false);
		zoomPlus   = new JButton(new PlusIcon()); zoomPlus.setMargin(new Insets(1,0,1,0)); add(zoomPlus); zoomPlus.addActionListener(buttonListener); //zoomPlus.setPreferredSize(new Dimension(19,19));

		layout.putConstraint(SpringLayout.NORTH, speed,      1, SpringLayout.NORTH, this);
		layout.putConstraint(SpringLayout.NORTH, speedMinus, 4, SpringLayout.SOUTH, speed);
		layout.putConstraint(SpringLayout.NORTH, speedPlus,  4, SpringLayout.SOUTH, speed);
		layout.putConstraint(SpringLayout.SOUTH, speedVal,  -1, SpringLayout.SOUTH, speedPlus);
		layout.putConstraint(SpringLayout.NORTH, speedMax,   0, SpringLayout.NORTH, speed);
				
		layout.putConstraint(SpringLayout.NORTH, timeLabel, 0, SpringLayout.NORTH, this);
		layout.putConstraint(SpringLayout.WEST, timeLabel, 0, SpringLayout.WEST, time);
		layout.putConstraint(SpringLayout.WEST, time, (int)(getPreferredSize().getWidth()/2)-50, SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.NORTH, time, 3, SpringLayout.SOUTH, timeLabel);

		layout.putConstraint(SpringLayout.NORTH, zoom,      1, SpringLayout.NORTH, speed);
		layout.putConstraint(SpringLayout.NORTH, zoomMinus, 4, SpringLayout.SOUTH, speed);
		layout.putConstraint(SpringLayout.NORTH, zoomPlus,  4, SpringLayout.SOUTH, speed);
		layout.putConstraint(SpringLayout.SOUTH, zoomVal,  -1, SpringLayout.SOUTH, zoomPlus);

		layout.putConstraint(SpringLayout.WEST, speed, 0, SpringLayout.WEST, speedMinus);
		layout.putConstraint(SpringLayout.EAST, speedMinus, 0, SpringLayout.WEST, speedVal);
		layout.putConstraint(SpringLayout.EAST, speedVal, 0, SpringLayout.WEST, speedPlus);
		layout.putConstraint(SpringLayout.EAST, speedPlus, -10, SpringLayout.WEST, time);
		layout.putConstraint(SpringLayout.WEST, speedMax, 5, SpringLayout.EAST, speed);

		layout.putConstraint(SpringLayout.WEST, zoom, 0, SpringLayout.WEST, zoomMinus);
		layout.putConstraint(SpringLayout.WEST, zoomMinus, 10, SpringLayout.EAST, time);
		layout.putConstraint(SpringLayout.WEST, zoomVal, 0, SpringLayout.EAST, zoomMinus);
		layout.putConstraint(SpringLayout.WEST, zoomPlus, 0, SpringLayout.EAST, zoomVal);
	}
	
	public void enableButtons(){
		speedMinus.setEnabled(sim.canSlowDown());
		if(sim.maxSpeed) speedVal.setText("MAX");
		else speedVal.setText(""+sim.getSpeed());
		speedPlus.setEnabled(sim.canSpeedUp());
		speedMax.setEnabled(!sim.maxSpeed);
		zoomMinus.setEnabled(sim.canZoomOut());
		zoomVal.setText(""+sim.getZoom());
		zoomPlus.setEnabled(sim.canZoomIn());
	}
	
	public void disableButtons(){
		speedMinus.setEnabled(false);
		speedPlus.setEnabled(false);
		speedMax.setEnabled(false);
		zoomMinus.setEnabled(false);
		zoomPlus.setEnabled(false);
	}

	public void paintComponent(Graphics g){
		super.paintComponent(g);
		time.setText(sim.state.getReadableTime());
	}
		
	public void updateSpeed(){ speedVal.setText(""+sim.getSpeed()); }
	
	private class buttonListener implements ActionListener{
		public void actionPerformed(ActionEvent e) {
			     if(e.getSource() == speedMinus){ sim.minusSpeed(); speedVal.setText(""+sim.getSpeed()); }
			else if(e.getSource() == speedPlus){ sim.plusSpeed(); speedVal.setText(""+sim.getSpeed()); }
			else if(e.getSource() == speedMax){ sim.maxSpeed(); speedVal.setText("MAX"); }
			
			else if(e.getSource() == zoomMinus){ sim.minusZoom(); zoomVal.setText(""+sim.getZoom()); }
			else if(e.getSource() == zoomPlus){ sim.plusZoom(); zoomVal.setText(""+sim.getZoom()); }
		}
	}
}
