/*
 * File:   BehaviourGraphWindow.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   04-03-05
 */
 
 package gui.graph;
 
import gui.*;
import gui.icon.PlotIcon;
import sim.Simulation;
import sim.livestock.*;
import main.PastureSim;

import java.awt.*;
import java.awt.Color;
import java.awt.Container;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.*;
import javax.swing.*;
import javax.swing.SpringLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;

/** Graphs animal behaviour */
public class BehaviourGraphWindow extends GraphWindow{

	public static final Color EAT_COLOUR    = Color.GREEN;
	public static final Color FORAGE_COLOUR = Color.YELLOW;
	public static final Color DRINK_COLOUR  = Color.BLUE;
	public static final Color FLOCK_COLOUR  = Color.MAGENTA;
	public static final Color SMEM_COLOUR   = Color.RED;
	public static final Color LMEM_COLOUR   = Color.ORANGE;
	public static final Color IDLE_COLOUR   = Color.GRAY;

	public BehaviourGraphWindow(UserInterface ui, Simulation sim){
		super(ui, sim, "Animal Behaviour");
	}
	
	
	/** Generate the gui */
	public void generate(){
		plots = new JCheckBox[7];
		plots[0] = new JCheckBox("Eating",                true);  plots[0].setIcon(new PlotIcon(EAT_COLOUR,    plots[0])); plots[0].setPreferredSize(new Dimension(170, 25)); plots[0].addActionListener(this);
		plots[1] = new JCheckBox("Foraging",              true);  plots[1].setIcon(new PlotIcon(FORAGE_COLOUR, plots[1])); plots[1].setPreferredSize(new Dimension(170, 25)); plots[1].addActionListener(this);
		plots[2] = new JCheckBox("Seeking Water",         true);  plots[2].setIcon(new PlotIcon(DRINK_COLOUR,  plots[2])); plots[2].setPreferredSize(new Dimension(170, 25)); plots[2].addActionListener(this);
		plots[3] = new JCheckBox("Flocking",              false); plots[3].setIcon(new PlotIcon(FLOCK_COLOUR,  plots[3])); plots[3].setPreferredSize(new Dimension(170, 25)); plots[3].addActionListener(this);
		plots[4] = new JCheckBox("Long Term Memory",      false); plots[4].setIcon(new PlotIcon(LMEM_COLOUR,   plots[4])); plots[4].setPreferredSize(new Dimension(170, 25)); plots[4].addActionListener(this);
		plots[5] = new JCheckBox("Short Term Memory",     false); plots[5].setIcon(new PlotIcon(SMEM_COLOUR,   plots[5])); plots[5].setPreferredSize(new Dimension(170, 25)); plots[5].addActionListener(this);
		plots[6] = new JCheckBox("Idle (Sleeping)",       false); plots[6].setIcon(new PlotIcon(IDLE_COLOUR,   plots[6])); plots[6].setPreferredSize(new Dimension(170, 25)); plots[6].addActionListener(this);
		super.generate();
	}
	
	
	public void drawGraph(Graphics g){
		if(sim.animalGraphData == null) return;
		if(plots[0].isSelected()){ g.setColor(EAT_COLOUR);    plotMinutes(sim.animalGraphData[0], g); }
		if(plots[1].isSelected()){ g.setColor(FORAGE_COLOUR); plotMinutes(sim.animalGraphData[1], g); }
		if(plots[2].isSelected()){ g.setColor(DRINK_COLOUR);  plotMinutes(sim.animalGraphData[2], g); }
		if(plots[3].isSelected()){ g.setColor(FLOCK_COLOUR);  plotMinutes(sim.animalGraphData[3], g); }
		if(plots[4].isSelected()){ g.setColor(LMEM_COLOUR);   plotMinutes(sim.animalGraphData[4], g); }
		if(plots[5].isSelected()){ g.setColor(SMEM_COLOUR);   plotMinutes(sim.animalGraphData[5], g); }
		if(plots[6].isSelected()){ g.setColor(IDLE_COLOUR);   plotMinutes(sim.animalGraphData[6], g); }
	}
}
