/*
 * File:   WaterAddActionPanel.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */

 
 
package gui.view;

import sim.*;
import sim.pasture.*;
import utils.*;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;


/** The action panel for adding water sources */
public class WaterAddActionPanel extends ActionPanel{

	private LinkedList polyPoints;
	private Point lastPoint;
	private WaterGeometry geom;
	private int proxLineX;
	private int proxLineY;
	private boolean draw;
	
	public WaterAddActionPanel(Simulation sim){
		super(sim);
		polyPoints = new LinkedList();
		geom = sim.getWaterGeometry();
		draw = true;
	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		sim.drawPasture(g);
		
		if(polyPoints.size() > 0) drawPolySoFar(g);
		g.setColor(Color.WHITE);
		if(lastPoint != null) sim.drawScaledLine(g, lastPoint.x, lastPoint.y, proxLineX, proxLineY);		
	}
	
	public void drawPolySoFar(Graphics g){
		Point last = new Point(0,0);
		boolean first = true;
		g.setColor(Color.WHITE);
		for(Iterator i = polyPoints.iterator(); i.hasNext();){
			if(first){ last = (Point)i.next(); first = false; }
			else{
				Point curPoint = (Point)i.next();
				int x = last.x;
				int y = last.y;
				sim.drawScaledLine(g, x, y, curPoint.x, curPoint.y);
				last = curPoint;
			}
		}
		
		sim.drawScaledLine(g, last.x, last.y, lastPoint.x, lastPoint.y);
	}
	
	
	public Polygon currentPoly(){	
		Point p;
		Polygon poly = new Polygon();
		for(Iterator i = polyPoints.iterator(); i.hasNext();){
			p = (Point)i.next();
			poly.addPoint(p.x, p.y);
		}
		if(lastPoint != null) poly.addPoint(lastPoint.x, lastPoint.y);
		return poly; 
	}public Polygon currentPoly(int x, int y){
		Polygon poly = currentPoly();
		poly.addPoint(x,y);
		return poly; 
	}
	
	public boolean selfOverlap(int x, int y){
		if(polyPoints.size() < 2) return false;
		Point p;
		Polygon poly = new Polygon();
		for(Iterator i = polyPoints.iterator(); i.hasNext();){
			p = (Point)i.next();
			poly.addPoint(p.x, p.y);
		}
		if(lastPoint != null) poly.addPoint(lastPoint.x, lastPoint.y);
		for(int i = 0; i < poly.npoints -1; i++){
			if(XLine2D.linesIntersect(
			   poly.xpoints[i], poly.ypoints[i], poly.xpoints[i+1], poly.ypoints[i+1],
			   lastPoint.x, lastPoint.y, x, y)) return true;
		}
		return false;
	}
	
	public boolean duplicatePoint(int x, int y){
		Point p = new Point(x,y);
		Polygon poly = new Polygon();
		for(Iterator i = polyPoints.iterator(); i.hasNext();){
			if(((Point)i.next()).equals(p)) return true;
		}
		if(lastPoint != null && lastPoint.equals(p)) return true;
		return false;
	}

	public void mouseSingleClicked(MouseEvent e){
		Point2D.Double p = sim.scaledMousePos(e.getX(), e.getY());
		Point p_ = geom.findFieldVertex(p);
		if(!geom.potentialOverlapFields(currentPoly(p_.x, p_.y))
		&& !selfOverlap(p_.x, p_.y)
		&& !duplicatePoint(p_.x, p_.y)){
			if(lastPoint != null) polyPoints.add(lastPoint);
			lastPoint = p_;
			repaint();
		}
	}

	public void mouseDoubleClicked(MouseEvent e){
		
		if(!geom.overlapFields(currentPoly(lastPoint.x, lastPoint.y))
		&& !selfOverlap(lastPoint.x, lastPoint.y)
		&& PathFinderMap.willBeDetected(sim, currentPoly(lastPoint.x, lastPoint.y))){
			if(polyPoints.size() > 1){
				Polygon poly = currentPoly();
				if(poly != null){
					FieldWater w = new FieldWater(poly, sim);
					sim.addWater(w);
				}
				polyPoints = new LinkedList();
				lastPoint = null;
				sim.setMode(Simulation.MODE_PASTURE);
				repaint();
			}
		}
	}
	
	public void mouseMoved(MouseEvent e){
		super.mouseMoved(e);
		Point2D.Double s = sim.scaledMousePos(mouseX, mouseY);
		Point closestVertex = geom.findFieldVertex(s);
		proxLineX = closestVertex.x;
		proxLineY = closestVertex.y;
		repaint();
	}

}
