/*
 * File:   PastureSim.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */



package main;

import sim.*;
import gui.*;
import utils.*;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;


/** The main driver class for the application.
  * It manages threads and multiple simulation windows. */
public class PastureSim{

	public static boolean printPastureData;   // Weather to show pasture data
	public static boolean printAnimalData;    // Weather to show animal data
	public static boolean textOnly;           // Weather we are in text only mode
	public static int     speed;              // The speed to use in text only mode
	
	private static BufferedWriter pStats;
	private static BufferedWriter aStats;
	
	
	private static LinkedList windows;
	public static String chosenName;
	public static int chosenArea;
	public static int chosenRate;
		
	public static void main(String[] args){
	
		
		try{
			File f = null;
			for(int i = 0; i < args.length; i++){
				     if(args[i].equals("-pdata")){ printPastureData = true; pStats = new BufferedWriter(new FileWriter(new File("pasture_data"))); }
				else if(args[i].equals("-adata")){ printAnimalData = true; aStats = new BufferedWriter(new FileWriter(new File("animal_data"))); }
				else if(args[i].equals("-data")){ printPastureData = true; printAnimalData = true;  pStats = new BufferedWriter(new FileWriter(new File("pasture_data")));  aStats = new BufferedWriter(new FileWriter(new File("animal_data"))); }
				else if(args[i].equals("-nw")) textOnly = true;
				else f = new File(args[i]);
			}
			
			if(textOnly && f.exists()){
				windows = new LinkedList();
				UserInterface ui = new UserInterface(f);
				windows.add(ui);
				while(true){
					synchronized(windows){
						if(windows.size() == 0) System.exit(0);
					}
				}
			}else textOnly = false;
		}catch(Exception e){ System.err.println("Usage: java PastureSim [-max|-pdata|-adata|-data|-nw|<file>]"); return; }
		
		try{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}catch(Exception e){ e.printStackTrace(); }
				
		windows = new LinkedList();
		UserInterface ui = new UserInterface();
		windows.add(ui);
		while(true){
			synchronized(windows){
				if(windows.size() == 0) System.exit(0);
			}
		}
	}
	
	
	
	public static void newSimulation(UserInterface ui){

		boolean success = false;
		while(!success){
			try{
				File arielPhotoFile = FileSelector.newSimulation(ui);
				if(arielPhotoFile == null) return;
				if(chosenArea < 4){
					gui.Popup.error("ERROR: The pasture must be at least 4 m\u00B2 in area.");
				}
				else if(chosenName.equals("")){
					gui.Popup.error("ERROR: You must supply a name for the pasture");
				}else success = true;			
				
				File simulationFile = new File(arielPhotoFile.getParent() + File.separator + chosenName);
				if(!simulationFile.getName().toLowerCase().endsWith(".psm"))
					simulationFile = new File(simulationFile.getPath() + ".psm");
				
				if(ui.isSimulationLoaded()){
					UserInterface newUI = new UserInterface(simulationFile,  arielPhotoFile, chosenArea, chosenRate);
					windows.add(newUI);
				}else{
					ui.loadSimulation(simulationFile, arielPhotoFile, chosenArea, chosenRate);
				}
			}catch(Exception e){ e.printStackTrace(); gui.Popup.error("ERROR: " + e); success = true; }
		}
	}
	
	
	public static void openSimulation(UserInterface ui){
		File simulationFile = FileSelector.openSimulation(ui);
		if(simulationFile == null){ return; }
		Simulation sim = null;		
		try {
			if(ui.isSimulationLoaded()){
				UserInterface newUI = new UserInterface(simulationFile);
				synchronized(windows){ windows.add(newUI); }
			}else{
				ui.loadSimulation(simulationFile);
			}
		}catch(Exception e){
			e.printStackTrace();
			gui.Popup.error("ERROR: Could not load simulation.");
			return;
		}		
	}
	
	public static void cleanUp(UserInterface ui){
		synchronized(windows){
			windows.remove(ui);
			if(windows.size() == 0){
				try{
					if(pStats != null){ pStats.flush(); }
					if(aStats != null){ aStats.flush(); }
				}catch(IOException e){ e.printStackTrace(); }
				System.exit(0);
			}
		}
	}
	
	
	public static void exit(){
		if(gui.Popup.verify("Are you sure you wish to completely exit the application")){
			// Close all the windwos
			UserInterface ui;
			while(true){
				synchronized(windows){
					ui = (UserInterface)windows.getFirst(); 
				}
				ui.close();
			}
		}
	}
	
	/** Write ainimal data */
	public static void writeAnimal(String s, long n){
		try{
			aStats.write(""+(float)n/1440.0f+" "+s);
		}catch(IOException e){ e.printStackTrace(); }
	}
	
	/** Write pasture data */
	public static void writePasture(String s, long n){
		try{
			pStats.write(""+(float)n/1440.0f+" "+s);
			pStats.flush(); aStats.flush();
		}catch(IOException e){ e.printStackTrace(); }
	}
}
