/*
 * File:   AnimalContainer.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */

 
package sim.pasture;

import gui.*;
import sim.pasture.*;
import sim.livestock.*;
import sim.*;
import utils.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;
import java.awt.geom.*;


/** Any object which contains animals implements this interface.
  * Current implementing classes are State and FieldBounds */
public interface AnimalContainer{

	
	// This establishes if the container owns the area
	public boolean ownsArea(Point2D.Double p, int size);

	// Returns any animal containers sharing this point through an open gate
	// The gate should be stored in WaterSources setGate method if a gate is found
	public AnimalContainer findOpenNeighbours(Point2D.Double p);
	
	
	// This tells us wheather the animal container holds point p
	public boolean holds(Point2D.Double p);
	
	// This tells us wheather the animal container fully holds the pixel at p
	public boolean fullyHolds(Point2D.Double p);
	
	/** Returns true if the animal containers are connected by an open gate through the line l*/
	public boolean connected(AnimalContainer ac, Line2D.Double l);
	
	
	/** Returns the open gate eventually leading the shortest path to p */
	public FieldGate findGate(Point2D.Double p);

	
	// Returns all dijkstra edges (gates) which are reachable
	// from the point p through this field, ordered by distance
	public TreeSet fringeEdges(Point2D.Double p);

	
	// Returns all dijkstra edges (gates) which are reachable
	// from the gate e through this field, ordered by distance
	public TreeSet fringeEdges(DijkstraEdge e, LinkedList visited);

	
	/** Returns weather the point p is on any of the field's gates */
	public boolean onGate(Point2D.Double p);
	
}
