/*
 * File:   GraphData.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   05-03-05
 */

 
package utils;

import java.util.Vector;
import java.io.Serializable;

 
/** Stores graph data. Reduces the resolution for older data */
public class GraphData implements Serializable{

	private Vector years;
	
	public GraphData(){
		years = new Vector();
	
/*		for(int i = 0; i < 400; i++){
			//for(int j = 0; j < 2; j++){
				updateMin(i*1440l+1, 0.4);
				updateMin(i*1440l+2, 0.8);
			//}
		}
	
		for(int i = 0; i < 40; i += 10){
			for(int j = 0; j < 2; j++){
				System.out.println(""+(i*1440l+j)+":"+getVal(i,j));
			}
		}
*/	}
	
	
	// Update a minutely value
	public void updateMin(long min, double val){
		int days = (int)(min/1440);
		int year = days/365;
		int yDay = days % 365;
		int dMin = (int)(min % 1440l);
		
//System.out.println(""+year+":"+yDay+":"+dMin+" - "+val);
		
		if(year >= years.size()){
			// Increase vector size
			years.add(new GraphYear(yDay, dMin, val));
		}
		else{
			// update year's minute value
			GraphYear y = (GraphYear)years.elementAt(year);
			y.update(year, yDay, dMin, val);
		}
	}		 
	
	// Update a daily value
	public void updateDay(long min, double val){
		int days = (int)(min/1440l);
		int year = days/365;
		int yDay = days % 365;
		
//System.out.println(""+year+":"+yDay+" - "+val);
		
		if(year >= years.size()){
			years.add(new GraphYear(yDay, val));
		}
		else{
			GraphYear y = (GraphYear)years.elementAt(year);
			if(y == null){}
			y.update(yDay, val);
		}
	}
	
	
	// Gets the value for a day
	public double getVal(int day){
		int year = day/365;
		int yDay = day % 365;
		if(year >= years.size() || yDay >= 365) return 0.0;
		return ((GraphYear)years.elementAt(year)).getVal(yDay);
	}
	
	// Gets the value for a minute
	public double getVal(int day, int min){
		if(min < 0 || min >= 1440) return 0.0;
		int year = day/365;
		int yDay = day % 365;
		if(year >= years.size() || yDay >= 365) return 0.0;
		return ((GraphYear)years.elementAt(year)).getVal(yDay, min);
	}
	
	
	/** Represents the data for a year */
	class GraphYear implements Serializable{
		
		private double average;  // The average value for this year
		private GraphDay[] days; // The values for each day of the year
		
		// Constructor for average year logger
		public GraphYear(double val){ average = val; days = null; }
		
		// Constructor for a day logger
		public GraphYear(int day, double val){
			average = val;
			days = new GraphDay[365];
			days[day] = new GraphDay(val);
		}
		
		// Constructor for a minute logger
		public GraphYear(int day, int min, double val){
			average = val;
			days = new GraphDay[365];
			days[day] = new GraphDay(min, val);
		}
		
		// Compress this daily data
		public void compress(){ days = null; }
		
		// Compress this daily data
		public void compressDay(int day){ if(days.length > day && days[day] != null) days[day].compress(); }
		
		// Returns wether it has been compressed
		public boolean isCompressed(){ return days == null; }
		
		// Update the data for a day logger
		public void update(int day, double val){
			average = average*(364.0/365.0) + val*(1.0/365.0);
			if(days != null){ days[day] = new GraphDay(val); }
		}
		
		// Update the data for a minute logger
		public void update(int year, int day, int min, double val){
			average = average*((365.0*1439.0)/(365.0*1440.0)) + val*(1.0/(365.0*1440.0));
			if(days != null){
//System.out.println("adding new min-day");
				if(days[day] == null) days[day] = new GraphDay(min, val);
				else days[day].update(min, val);
				
				if(day >= 10 && days[day-10] != null) days[day-10].compress();
				else if(year > 0){ ((GraphYear)years.elementAt(year-1)).compressDay(365+day-10); }
			}
		}
		
		// Gets a value for a day, returns average if it has been compressed
		public double getVal(int day){
			if(days == null || days[day] == null) return average;
			return days[day].getAvg();
		}
		
		// Gets a value for a minute, returns average if it has been compressed
		public double getVal(int day, int min){
			if(days == null || days[day] == null) return average;
			return days[day].getVal(min);
		}
		
		// Returns the average
		public double getAvg(){ return average; }
		
		// Set the average value
		public void setAvg(double val){ average = val; }
	}
	
	
	class GraphDay implements Serializable{
		
		private double average;   // The average value for this day
		private double[] minutes; // The values for each minute of the day
		
		// Constructor for average day logger
		public GraphDay(double val){ average = val; minutes = null; }
		
		// Constructor for detailed minute logger
		public GraphDay(int min, double val){
			average = val;
			minutes = new double[1440]; minutes[min] = val;
//System.out.println("adding min-day data");
		}
		
		// Compress this daily data
		public void compress(){ minutes = null;	}
		
		// Returns wether it has been compressed
		public boolean isCompressed(){ return minutes == null; }
		
		
		// Update the minute data
		public void update(int min, double val){
			average = average*(1439.0/1440.0) + val*(1.0/1440.0);
			if(minutes != null){ minutes[min] = val; }
		}
		
		// Gets a spacific value, returns average if it has been compressed
		public double getVal(int min){
			if(minutes == null) return average;
			return minutes[min];
		}
		
		// Returns the average
		public double getAvg(){ return average; }
		
		// Set the average value
		public void setAvg(double val){ average = val; }
	}
}
