/*
 * File:   WaterGeometry.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */


package utils;

import gui.*;
import sim.*;
import sim.pasture.*;
import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.geom.*;


/** Performs all calculations associated with the water soruces */
public class WaterGeometry implements Serializable{
	
	public LinkedList water;
	public LinkedList fields;
	public Simulation sim;
	
	public WaterGeometry(LinkedList water, LinkedList fields, Simulation sim){
		this.water = water;
		this.fields = fields;
		this.sim = sim;
	}

	public boolean overlapPointsFields(Polygon p){
		if(p == null) return false;
		for(Iterator i = water.iterator(); i.hasNext();){
			if(((FieldWater)i.next()).overlapPoints(p)) return true;
		}
		return false;
	}
	public boolean potentialOverlapFields(Polygon p){
		if(p == null) return false;
		for(Iterator i = water.iterator(); i.hasNext();){
			if(((FieldWater)i.next()).potentialOverlap(p)) return true;
		}
		return false;
	}
	public boolean overlapFields(Polygon p){
		if(p == null) return false;
		for(Iterator i = water.iterator(); i.hasNext();){
			if(((FieldWater)i.next()).overlap(p)) return true;
		}
		return false;
	}
	
	
	/** Finds the closest point of the closest water area */
	public Point2D.Double findClosestWater(Point2D.Double p){
		AnimalContainer f = sim.getFieldGeometry().findContainingField(p);
		if(f instanceof State){
//System.out.println("no field bounds");
			// Find closest point not in any fields
			return findClosestPoint(p);
		}else{
			//get list of waters in field
			//LinkedList availWater = f.findAvailableWater(water);
			// If no water accessible return no point
			//if(availWater.size() == 0) return null;
			// Find closest point for water available
			// Remember to ensure point is within field.
			return findClosestPoint(p, (FieldBounds)f);
		}
	}

	
	/** Finds the closest point to (x,w) in the waters w which 
	 *  does not intersect any fields */
	public Point2D.Double findClosestPoint(Point2D.Double pt){
		
		Point2D.Double p = null;  // Closest point so far
		double d = Double.MAX_VALUE; // Smallest Distance
		boolean found = false;
		FieldGeometry geom = sim.getFieldGeometry();
		// Search for the closest water edge
//System.out.println("water size = " + water.size());		
		for(Iterator i = water.iterator(); i.hasNext();){
			Point2D.Double p_ = ((FieldWater)i.next()).centre; // centre of the water
			double d_ = p_.distance(pt);
//System.out.println("intersects?"+geom.intersectsFields(x, y, p_.x, p_.y));
			if(d_ < d && !geom.intersectsFields(new Point((int)(pt.x+0.5), (int)(pt.y+0.5)), new Point((int)(p_.x+0.5), (int)(p_.y+0.5)))){
				found = true;
				d = d_;
				p = p_;
			}
		}
		return p; // return the point
	}

		
	/** Finds the closest point to pt in the waters w which is 
	 * contained within the field f */
	public Point2D.Double findClosestPoint(Point2D.Double pt, FieldBounds f){
		
		Point2D.Double p = null;  // Closest point so far
		double d = Double.MAX_VALUE; // Smallest Distance
		boolean found = false;
		// Search for the closest fence
		for(Iterator i = water.iterator(); i.hasNext();){
			Point2D.Double p_ = ((FieldWater)i.next()).centre; // centre of the water
			double d_ = p_.distance(pt);
			if(d_ < d && !f.intersects(new Point((int)(pt.x+0.5), (int)(pt.y+0.5)), new Point((int)(p_.x+0.5), (int)(p_.y+0.5)))){
				found = true;
				d = d_;
				p = p_;
			}
		}
		return p;
	}
		

	/** Finds the closest point to (x,w) in the waters w which 
	 *  does not intersect any fields */
/*	public Point2D.Double findClosestPoint(int x, int y, LinkedList w){
		
		Point p = null;  // Closest point so far
		double d = Double.MAX_VALUE; // Smallest Distance
		boolean found = false;
		FieldGeometry geom = sim.getFieldGeometry();
		// Search for the closest water edge
		for(Iterator i = water.iterator(); i.hasNext();){
			Point p_ = new Point(0,0); // Closest point for this field
			double d_ = ((FieldWater)i.next()).getClosestVertex(x, y, p_); // Smallest dist for that field
			if(d_ < d && !geom.intersectsFields(x, y, p_.x, p_.y)){
				found = true;
				d = d_;
				p = p_;
			}
		}
		if(p == null) return null; // Prevents a nullpointerexception for next statement
		return new Point2D.Double(p.x, p.y); // return the point
	}
*/

	
	/** Finds the closest point to (x,w) in the waters w which is 
	 * contained within the field f */
/*	public Point2D.Double findClosestPoint(int x, int y, LinkedList w, FieldBounds f){
		
		Point p1 = null; Point p2 = null; // Closest fence so far
		double d = Double.MAX_VALUE; // Smallest Distance
		boolean found = false;
		// Search for the closest fence
		for(Iterator i = water.iterator(); i.hasNext();){
			Point p1_ = new Point(0,0); Point p2_ = new Point(0,0); // Closest fence for this field
			double d_ = ((FieldWater)i.next()).getClosestEdge(x, y, p1_, p2_, f); // Smallest dist for that field
			if(d_ < d){
				found = true;
				d = d_;
				p1 = p1_; p2 = p2_;
			}
		}
		if(!found) return null; // This should never hapen if water exists
	
		Vector2D v = new Vector2D(p1.x, p1.y, p2.x, p2.y);
		Vector2D vp = v.perp();
		Point2D.Double p = vp.project(new Point2D.Double(x,y), d);
		 // If point fits onto line segment and is in field then return
		if(!f.intersects(x, y, p.x, p.y) && insideLine(p, p1.x, p1.y, p2.x, p2.y)) return p;
			
		// If point is past line segment then the point is one of the line's vertices
		if(p1.distance(new Point(x,y)) < p2.distance(new Point(x,y)) && !f.intersects(x, y, p1.x, p1.y))
			return new Point2D.Double(p1.x, p1.y);
		return new Point2D.Double(p2.x, p2.y);
	}
*/
		
	/** Finds the closest vertex on the polygon to (x,y) */
	public Point findFieldVertex(Point2D.Double pt){
		Point closestP = new Point((int)(pt.x+0.5), (int)(pt.y+0.5));
		double smallestDistance = sim.SNAP_RADIUS;
		for(Iterator i = water.iterator(); i.hasNext();){
			Point p = ((FieldWater)i.next()).getCloseVertex(pt);
			if(p != null){
				double dist = Point2D.distance(closestP.x, closestP.y, p.x, p.y); 
				if(dist < smallestDistance){
					//System.out.println("Smallun found!");
					smallestDistance = dist;
					closestP = p;
				}
			}
		}
		return closestP;
	}
	
	public FieldWater findContainingFieldDraw(Point2D.Double pt){
		for(Iterator i = water.iterator(); i.hasNext();){
			FieldWater f = (FieldWater)i.next();
			if(f.contains(pt)) return f;
		}
		return null;
	}

	// Checks wheather the point (x,y) is in any water
	public boolean inWater(Point2D.Double p){
		for(Iterator i = water.iterator(); i.hasNext();){
			if(((FieldWater)i.next()).contains(p)) return true;
		}
		return false;
	}
	
	// Checks if the point p is on the line segment
	public boolean insideLine(Point2D.Double p, double x1, double y1, double x2, double y2){
		return ((p.x > x1 && p.x < x2) || (p.x > x2 && p.x < x1))
			&& ((p.y > y1 && p.y < y2) || (p.y > y2 && p.y < y1));
	}
	
	
	//private void writeObject(java.io.ObjectOutputStream out) throws IOException{}
	//private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException{}

}
