/*
 * File:   WaterSource.java
 * Author: Conrad Rider
 * Email:  c.rider@eviltree.co.uk
 * Date:   22-02-05
 */


package utils;

import gui.*;
import sim.*;
import sim.pasture.*;
import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.geom.*;


/** Represents a pixel which has access to water. Used in
  * PathFinderMap.java for path finding. */
public class WaterSource implements Comparable{
	

	private Point p; // The position of this source
	private AnimalContainer ac; // The container which owns this source
	private float dist; // The distance of the water source
	
	// Construct a primary source
	public WaterSource(Point p, AnimalContainer ac, float dist){
		this.p = p;
		this.ac = ac;
		this.dist = dist;
	}
	
	// Accessor methods
	public Point getPos(){ return p; }
	public AnimalContainer getContainer(){ return ac; }
	public float dist(){ return dist; }
	
	public int compareTo(Object o){
		if(o instanceof WaterSource){
			if(dist > ((WaterSource)o).dist()) return  1;
			if(dist < ((WaterSource)o).dist()) return -1;
			if(dist == ((WaterSource)o).dist() && this.equals(o)) return 0;
			
			// If this is the case then distances are equal, but they are
			// not the same element and we need to find another criteria
			// to order the elements. Here i use their location
			
			if(p.x > ((WaterSource)o).getPos().x) return  1;
			if(p.x < ((WaterSource)o).getPos().x) return -1;
			if(p.y > ((WaterSource)o).getPos().y) return  1;
			if(p.y < ((WaterSource)o).getPos().y) return -1;
			
			// If we reach here then they are on the same point
			// and must be the same, so fall out of the if statement
		}
		return 0;
	}
	
}
